#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import implementationguide
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class ImplementationGuideTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("ImplementationGuide", js["resourceType"])
        return implementationguide.ImplementationGuide(js)
    
    def testImplementationGuide1(self):
        inst = self.instantiate_from("implementationguide-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a ImplementationGuide instance")
        self.implImplementationGuide1(inst)
        
        js = inst.as_json()
        self.assertEqual("ImplementationGuide", js["resourceType"])
        inst2 = implementationguide.ImplementationGuide(js)
        self.implImplementationGuide1(inst2)
    
    def implImplementationGuide1(self, inst):
        self.assertEqual(force_bytes(inst.contact[0].name), force_bytes("ONC"))
        self.assertEqual(force_bytes(inst.contact[0].telecom[0].system), force_bytes("url"))
        self.assertEqual(force_bytes(inst.contact[0].telecom[0].value), force_bytes("http://www.healthit.gov"))
        self.assertEqual(force_bytes(inst.contact[1].name), force_bytes("HL7"))
        self.assertEqual(force_bytes(inst.contact[1].telecom[0].system), force_bytes("url"))
        self.assertEqual(force_bytes(inst.contact[1].telecom[0].value), force_bytes("http://hl7.org/fhir"))
        self.assertEqual(force_bytes(inst.copyright), force_bytes("Published by ONC under the standard FHIR license (CC0)"))
        self.assertEqual(inst.date.date, FHIRDate("2015-01-01").date)
        self.assertEqual(inst.date.as_json(), "2015-01-01")
        self.assertEqual(force_bytes(inst.definition.grouping[0].description), force_bytes("Base package (not broken up into multiple packages)"))
        self.assertEqual(force_bytes(inst.definition.grouping[0].name), force_bytes("test"))
        self.assertEqual(force_bytes(inst.definition.page.generation), force_bytes("html"))
        self.assertEqual(force_bytes(inst.definition.page.nameUrl), force_bytes("patient-example.html"))
        self.assertEqual(force_bytes(inst.definition.page.page[0].generation), force_bytes("html"))
        self.assertEqual(force_bytes(inst.definition.page.page[0].nameUrl), force_bytes("list.html"))
        self.assertEqual(force_bytes(inst.definition.page.page[0].title), force_bytes("Value Set Page"))
        self.assertEqual(force_bytes(inst.definition.page.title), force_bytes("Example Patient Page"))
        self.assertEqual(force_bytes(inst.definition.parameter[0].code), force_bytes("apply"))
        self.assertEqual(force_bytes(inst.definition.parameter[0].value), force_bytes("version"))
        self.assertEqual(force_bytes(inst.definition.resource[0].description), force_bytes("A test example to show how an implementation guide works"))
        self.assertEqual(force_bytes(inst.definition.resource[0].exampleCanonical), force_bytes("http://hl7.org/fhir/us/core/StructureDefinition/patient"))
        self.assertEqual(force_bytes(inst.definition.resource[0].name), force_bytes("Test Example"))
        self.assertEqual(force_bytes(inst.dependsOn[0].uri), force_bytes("http://hl7.org/fhir/ImplementationGuide/uscore"))
        self.assertFalse(inst.experimental)
        self.assertEqual(force_bytes(inst.fhirVersion[0]), force_bytes("4.0.0"))
        self.assertEqual(force_bytes(inst.global_fhir[0].profile), force_bytes("http://hl7.org/fhir/us/core/StructureDefinition/patient"))
        self.assertEqual(force_bytes(inst.global_fhir[0].type), force_bytes("Patient"))
        self.assertEqual(force_bytes(inst.id), force_bytes("example"))
        self.assertEqual(force_bytes(inst.jurisdiction[0].coding[0].code), force_bytes("US"))
        self.assertEqual(force_bytes(inst.jurisdiction[0].coding[0].system), force_bytes("urn:iso:std:iso:3166"))
        self.assertEqual(force_bytes(inst.license), force_bytes("CC0-1.0"))
        self.assertEqual(force_bytes(inst.manifest.image[0]), force_bytes("fhir.png"))
        self.assertEqual(force_bytes(inst.manifest.other[0]), force_bytes("fhir.css"))
        self.assertEqual(force_bytes(inst.manifest.page[0].anchor[0]), force_bytes("patient-test"))
        self.assertEqual(force_bytes(inst.manifest.page[0].anchor[1]), force_bytes("tx"))
        self.assertEqual(force_bytes(inst.manifest.page[0].anchor[2]), force_bytes("uml"))
        self.assertEqual(force_bytes(inst.manifest.page[0].name), force_bytes("patient-test.html"))
        self.assertEqual(force_bytes(inst.manifest.page[0].title), force_bytes("Test Patient Example"))
        self.assertEqual(force_bytes(inst.manifest.rendering), force_bytes("http://hl7.org/fhir/us/daf"))
        self.assertEqual(force_bytes(inst.manifest.resource[0].exampleCanonical), force_bytes("http://hl7.org/fhir/us/core/StructureDefinition/patient"))
        self.assertEqual(force_bytes(inst.manifest.resource[0].relativePath), force_bytes("patient-test.html#patient-test"))
        self.assertEqual(force_bytes(inst.name), force_bytes("Data Access Framework (DAF)"))
        self.assertEqual(force_bytes(inst.packageId), force_bytes("hl7.fhir.us.daf"))
        self.assertEqual(force_bytes(inst.publisher), force_bytes("ONC / HL7 Joint project"))
        self.assertEqual(force_bytes(inst.status), force_bytes("draft"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.url), force_bytes("http://hl7.org/fhir/us/daf"))
        self.assertEqual(force_bytes(inst.version), force_bytes("0"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Contract) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Contract(domainresource.DomainResource):
    """ Legal Agreement.

    Legally enforceable, formally recorded unilateral or bilateral directive
    i.e., a policy or agreement.
    """

    resource_type = "Contract"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.alias = None
        """ Acronym or short name.
        List of `str` items. """

        self.applies = None
        """ Effective time.
        Type `Period` (represented as `dict` in JSON). """

        self.author = None
        """ Source of Contract.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.authority = None
        """ Authority under which this Contract has standing.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.contentDefinition = None
        """ Contract precursor content.
        Type `ContractContentDefinition` (represented as `dict` in JSON). """

        self.contentDerivative = None
        """ Content derived from the basal information.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.domain = None
        """ A sphere of control governed by an authoritative jurisdiction,
        organization, or person.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.expirationType = None
        """ Contract cessation cause.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.friendly = None
        """ Contract Friendly Language.
        List of `ContractFriendly` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Contract number.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.instantiatesCanonical = None
        """ Source Contract Definition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.instantiatesUri = None
        """ External Contract Definition.
        Type `str`. """

        self.issued = None
        """ When this Contract was issued.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.legal = None
        """ Contract Legal Language.
        List of `ContractLegal` items (represented as `dict` in JSON). """

        self.legalState = None
        """ Negotiation status.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.legallyBindingAttachment = None
        """ Binding Contract.
        Type `Attachment` (represented as `dict` in JSON). """

        self.legallyBindingReference = None
        """ Binding Contract.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.name = None
        """ Computer friendly designation.
        Type `str`. """

        self.relevantHistory = None
        """ Key event in Contract History.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.rule = None
        """ Computable Contract Language.
        List of `ContractRule` items (represented as `dict` in JSON). """

        self.scope = None
        """ Range of Legal Concerns.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.signer = None
        """ Contract Signatory.
        List of `ContractSigner` items (represented as `dict` in JSON). """

        self.site = None
        """ Specific Location.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.status = None
        """ draft | active | suspended | cancelled | completed | entered-in-
        error | unknown.
        Type `str`. """

        self.subType = None
        """ Subtype within the context of type.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.subject = None
        """ Contract Target Entity.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.subtitle = None
        """ Subordinate Friendly name.
        Type `str`. """

        self.supportingInfo = None
        """ Extra Information.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.term = None
        """ Contract Term List.
        List of `ContractTerm` items (represented as `dict` in JSON). """

        self.title = None
        """ Human Friendly name.
        Type `str`. """

        self.topicCodeableConcept = None
        """ Focus of contract interest.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.topicReference = None
        """ Focus of contract interest.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.type = None
        """ Legal instrument category.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.url = None
        """ Basal definition.
        Type `str`. """

        self.version = None
        """ Business edition.
        Type `str`. """

        super(Contract, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Contract, self).elementProperties()
        js.extend([
            ("alias", "alias", str, "string", True, None, False),
            ("applies", "applies", period.Period, "Period", False, None, False),
            ("author", "author", fhirreference.FHIRReference, "Reference", False, None, False),
            ("authority", "authority", fhirreference.FHIRReference, "Reference", True, None, False),
            ("contentDefinition", "contentDefinition", ContractContentDefinition, "ContractContentDefinition", False, None, False),
            ("contentDerivative", "contentDerivative", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("domain", "domain", fhirreference.FHIRReference, "Reference", True, None, False),
            ("expirationType", "expirationType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("friendly", "friendly", ContractFriendly, "ContractFriendly", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("instantiatesCanonical", "instantiatesCanonical", fhirreference.FHIRReference, "Reference", False, None, False),
            ("instantiatesUri", "instantiatesUri", str, "uri", False, None, False),
            ("issued", "issued", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("legal", "legal", ContractLegal, "ContractLegal", True, None, False),
            ("legalState", "legalState", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("legallyBindingAttachment", "legallyBindingAttachment", attachment.Attachment, "Attachment", False, "legallyBinding", False),
            ("legallyBindingReference", "legallyBindingReference", fhirreference.FHIRReference, "Reference", False, "legallyBinding", False),
            ("name", "name", str, "string", False, None, False),
            ("relevantHistory", "relevantHistory", fhirreference.FHIRReference, "Reference", True, None, False),
            ("rule", "rule", ContractRule, "ContractRule", True, None, False),
            ("scope", "scope", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("signer", "signer", ContractSigner, "ContractSigner", True, None, False),
            ("site", "site", fhirreference.FHIRReference, "Reference", True, None, False),
            ("status", "status", str, "code", False, None, False),
            ("subType", "subType", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", True, None, False),
            ("subtitle", "subtitle", str, "string", False, None, False),
            ("supportingInfo", "supportingInfo", fhirreference.FHIRReference, "Reference", True, None, False),
            ("term", "term", ContractTerm, "ContractTerm", True, None, False),
            ("title", "title", str, "string", False, None, False),
            ("topicCodeableConcept", "topicCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "topic", False),
            ("topicReference", "topicReference", fhirreference.FHIRReference, "Reference", False, "topic", False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("url", "url", str, "uri", False, None, False),
            ("version", "version", str, "string", False, None, False),
        ])
        return js


from . import backboneelement

class ContractContentDefinition(backboneelement.BackboneElement):
    """ Contract precursor content.

    Precusory content developed with a focus and intent of supporting the
    formation a Contract instance, which may be associated with and
    transformable into a Contract.
    """

    resource_type = "ContractContentDefinition"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.copyright = None
        """ Publication Ownership.
        Type `str`. """

        self.publicationDate = None
        """ When published.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.publicationStatus = None
        """ draft | active | retired | unknown.
        Type `str`. """

        self.publisher = None
        """ Publisher Entity.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.subType = None
        """ Detailed Content Type Definition.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.type = None
        """ Content structure and use.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractContentDefinition, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractContentDefinition, self).elementProperties()
        js.extend([
            ("copyright", "copyright", str, "markdown", False, None, False),
            ("publicationDate", "publicationDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("publicationStatus", "publicationStatus", str, "code", False, None, True),
            ("publisher", "publisher", fhirreference.FHIRReference, "Reference", False, None, False),
            ("subType", "subType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


class ContractFriendly(backboneelement.BackboneElement):
    """ Contract Friendly Language.

    The "patient friendly language" versionof the Contract in whole or in
    parts. "Patient friendly language" means the representation of the Contract
    and Contract Provisions in a manner that is readily accessible and
    understandable by a layperson in accordance with best practices for
    communication styles that ensure that those agreeing to or signing the
    Contract understand the roles, actions, obligations, responsibilities, and
    implication of the agreement.
    """

    resource_type = "ContractFriendly"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contentAttachment = None
        """ Easily comprehended representation of this Contract.
        Type `Attachment` (represented as `dict` in JSON). """

        self.contentReference = None
        """ Easily comprehended representation of this Contract.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ContractFriendly, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractFriendly, self).elementProperties()
        js.extend([
            ("contentAttachment", "contentAttachment", attachment.Attachment, "Attachment", False, "content", True),
            ("contentReference", "contentReference", fhirreference.FHIRReference, "Reference", False, "content", True),
        ])
        return js


class ContractLegal(backboneelement.BackboneElement):
    """ Contract Legal Language.

    List of Legal expressions or representations of this Contract.
    """

    resource_type = "ContractLegal"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contentAttachment = None
        """ Contract Legal Text.
        Type `Attachment` (represented as `dict` in JSON). """

        self.contentReference = None
        """ Contract Legal Text.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ContractLegal, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractLegal, self).elementProperties()
        js.extend([
            ("contentAttachment", "contentAttachment", attachment.Attachment, "Attachment", False, "content", True),
            ("contentReference", "contentReference", fhirreference.FHIRReference, "Reference", False, "content", True),
        ])
        return js


class ContractRule(backboneelement.BackboneElement):
    """ Computable Contract Language.

    List of Computable Policy Rule Language Representations of this Contract.
    """

    resource_type = "ContractRule"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contentAttachment = None
        """ Computable Contract Rules.
        Type `Attachment` (represented as `dict` in JSON). """

        self.contentReference = None
        """ Computable Contract Rules.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ContractRule, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractRule, self).elementProperties()
        js.extend([
            ("contentAttachment", "contentAttachment", attachment.Attachment, "Attachment", False, "content", True),
            ("contentReference", "contentReference", fhirreference.FHIRReference, "Reference", False, "content", True),
        ])
        return js


class ContractSigner(backboneelement.BackboneElement):
    """ Contract Signatory.

    Parties with legal standing in the Contract, including the principal
    parties, the grantor(s) and grantee(s), which are any person or
    organization bound by the contract, and any ancillary parties, which
    facilitate the execution of the contract such as a notary or witness.
    """

    resource_type = "ContractSigner"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.party = None
        """ Contract Signatory Party.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.signature = None
        """ Contract Documentation Signature.
        List of `Signature` items (represented as `dict` in JSON). """

        self.type = None
        """ Contract Signatory Role.
        Type `Coding` (represented as `dict` in JSON). """

        super(ContractSigner, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractSigner, self).elementProperties()
        js.extend([
            ("party", "party", fhirreference.FHIRReference, "Reference", False, None, True),
            ("signature", "signature", signature.Signature, "Signature", True, None, True),
            ("type", "type", coding.Coding, "Coding", False, None, True),
        ])
        return js


class ContractTerm(backboneelement.BackboneElement):
    """ Contract Term List.

    One or more Contract Provisions, which may be related and conveyed as a
    group, and may contain nested groups.
    """

    resource_type = "ContractTerm"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.action = None
        """ Entity being ascribed responsibility.
        List of `ContractTermAction` items (represented as `dict` in JSON). """

        self.applies = None
        """ Contract Term Effective Time.
        Type `Period` (represented as `dict` in JSON). """

        self.asset = None
        """ Contract Term Asset List.
        List of `ContractTermAsset` items (represented as `dict` in JSON). """

        self.group = None
        """ Nested Contract Term Group.
        List of `ContractTerm` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Contract Term Number.
        Type `Identifier` (represented as `dict` in JSON). """

        self.issued = None
        """ Contract Term Issue Date Time.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.offer = None
        """ Context of the Contract term.
        Type `ContractTermOffer` (represented as `dict` in JSON). """

        self.securityLabel = None
        """ Protection for the Term.
        List of `ContractTermSecurityLabel` items (represented as `dict` in JSON). """

        self.subType = None
        """ Contract Term Type specific classification.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.text = None
        """ Term Statement.
        Type `str`. """

        self.topicCodeableConcept = None
        """ Term Concern.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.topicReference = None
        """ Term Concern.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.type = None
        """ Contract Term Type or Form.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractTerm, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTerm, self).elementProperties()
        js.extend([
            ("action", "action", ContractTermAction, "ContractTermAction", True, None, False),
            ("applies", "applies", period.Period, "Period", False, None, False),
            ("asset", "asset", ContractTermAsset, "ContractTermAsset", True, None, False),
            ("group", "group", ContractTerm, "ContractTerm", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", False, None, False),
            ("issued", "issued", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("offer", "offer", ContractTermOffer, "ContractTermOffer", False, None, True),
            ("securityLabel", "securityLabel", ContractTermSecurityLabel, "ContractTermSecurityLabel", True, None, False),
            ("subType", "subType", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("text", "text", str, "string", False, None, False),
            ("topicCodeableConcept", "topicCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "topic", False),
            ("topicReference", "topicReference", fhirreference.FHIRReference, "Reference", False, "topic", False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


class ContractTermAction(backboneelement.BackboneElement):
    """ Entity being ascribed responsibility.

    An actor taking a role in an activity for which it can be assigned some
    degree of responsibility for the activity taking place.
    """

    resource_type = "ContractTermAction"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.context = None
        """ Episode associated with action.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.contextLinkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.doNotPerform = None
        """ True if the term prohibits the  action.
        Type `bool`. """

        self.intent = None
        """ Purpose for the Contract Term Action.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.linkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.note = None
        """ Comments about the action.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.occurrenceDateTime = None
        """ When action happens.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.occurrencePeriod = None
        """ When action happens.
        Type `Period` (represented as `dict` in JSON). """

        self.occurrenceTiming = None
        """ When action happens.
        Type `Timing` (represented as `dict` in JSON). """

        self.performer = None
        """ Actor that wil execute (or not) the action.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.performerLinkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.performerRole = None
        """ Competency of the performer.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.performerType = None
        """ Kind of service performer.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reason = None
        """ Why action is to be performed.
        List of `str` items. """

        self.reasonCode = None
        """ Why is action (not) needed?.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reasonLinkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.reasonReference = None
        """ Why is action (not) needed?.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.requester = None
        """ Who asked for action.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.requesterLinkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.securityLabelNumber = None
        """ Action restriction numbers.
        List of `int` items. """

        self.status = None
        """ State of the action.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.subject = None
        """ Entity of the action.
        List of `ContractTermActionSubject` items (represented as `dict` in JSON). """

        self.type = None
        """ Type or form of the action.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractTermAction, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermAction, self).elementProperties()
        js.extend([
            ("context", "context", fhirreference.FHIRReference, "Reference", False, None, False),
            ("contextLinkId", "contextLinkId", str, "string", True, None, False),
            ("doNotPerform", "doNotPerform", bool, "boolean", False, None, False),
            ("intent", "intent", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("linkId", "linkId", str, "string", True, None, False),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("occurrenceDateTime", "occurrenceDateTime", fhirdate.FHIRDate, "dateTime", False, "occurrence", False),
            ("occurrencePeriod", "occurrencePeriod", period.Period, "Period", False, "occurrence", False),
            ("occurrenceTiming", "occurrenceTiming", timing.Timing, "Timing", False, "occurrence", False),
            ("performer", "performer", fhirreference.FHIRReference, "Reference", False, None, False),
            ("performerLinkId", "performerLinkId", str, "string", True, None, False),
            ("performerRole", "performerRole", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("performerType", "performerType", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reason", "reason", str, "string", True, None, False),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reasonLinkId", "reasonLinkId", str, "string", True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, "Reference", True, None, False),
            ("requester", "requester", fhirreference.FHIRReference, "Reference", True, None, False),
            ("requesterLinkId", "requesterLinkId", str, "string", True, None, False),
            ("securityLabelNumber", "securityLabelNumber", int, "unsignedInt", True, None, False),
            ("status", "status", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
            ("subject", "subject", ContractTermActionSubject, "ContractTermActionSubject", True, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


class ContractTermActionSubject(backboneelement.BackboneElement):
    """ Entity of the action.
    """

    resource_type = "ContractTermActionSubject"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.reference = None
        """ Entity of the action.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.role = None
        """ Role type of the agent.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractTermActionSubject, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermActionSubject, self).elementProperties()
        js.extend([
            ("reference", "reference", fhirreference.FHIRReference, "Reference", True, None, True),
            ("role", "role", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


class ContractTermAsset(backboneelement.BackboneElement):
    """ Contract Term Asset List.
    """

    resource_type = "ContractTermAsset"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.answer = None
        """ Response to assets.
        List of `ContractTermOfferAnswer` items (represented as `dict` in JSON). """

        self.condition = None
        """ Quality desctiption of asset.
        Type `str`. """

        self.context = None
        """ Circumstance of the asset.
        List of `ContractTermAssetContext` items (represented as `dict` in JSON). """

        self.linkId = None
        """ Pointer to asset text.
        List of `str` items. """

        self.period = None
        """ Time period of the asset.
        List of `Period` items (represented as `dict` in JSON). """

        self.periodType = None
        """ Asset availability types.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.relationship = None
        """ Kinship of the asset.
        Type `Coding` (represented as `dict` in JSON). """

        self.scope = None
        """ Range of asset.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.securityLabelNumber = None
        """ Asset restriction numbers.
        List of `int` items. """

        self.subtype = None
        """ Asset sub-category.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.text = None
        """ Asset clause or question text.
        Type `str`. """

        self.type = None
        """ Asset category.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.typeReference = None
        """ Associated entities.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.usePeriod = None
        """ Time period.
        List of `Period` items (represented as `dict` in JSON). """

        self.valuedItem = None
        """ Contract Valued Item List.
        List of `ContractTermAssetValuedItem` items (represented as `dict` in JSON). """

        super(ContractTermAsset, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermAsset, self).elementProperties()
        js.extend([
            ("answer", "answer", ContractTermOfferAnswer, "ContractTermOfferAnswer", True, None, False),
            ("condition", "condition", str, "string", False, None, False),
            ("context", "context", ContractTermAssetContext, "ContractTermAssetContext", True, None, False),
            ("linkId", "linkId", str, "string", True, None, False),
            ("period", "period", period.Period, "Period", True, None, False),
            ("periodType", "periodType", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("relationship", "relationship", coding.Coding, "Coding", False, None, False),
            ("scope", "scope", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("securityLabelNumber", "securityLabelNumber", int, "unsignedInt", True, None, False),
            ("subtype", "subtype", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("text", "text", str, "string", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("typeReference", "typeReference", fhirreference.FHIRReference, "Reference", True, None, False),
            ("usePeriod", "usePeriod", period.Period, "Period", True, None, False),
            ("valuedItem", "valuedItem", ContractTermAssetValuedItem, "ContractTermAssetValuedItem", True, None, False),
        ])
        return js


class ContractTermAssetContext(backboneelement.BackboneElement):
    """ Circumstance of the asset.
    """

    resource_type = "ContractTermAssetContext"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ Codeable asset context.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reference = None
        """ Creator,custodian or owner.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.text = None
        """ Context description.
        Type `str`. """

        super(ContractTermAssetContext, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermAssetContext, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reference", "reference", fhirreference.FHIRReference, "Reference", False, None, False),
            ("text", "text", str, "string", False, None, False),
        ])
        return js


class ContractTermAssetValuedItem(backboneelement.BackboneElement):
    """ Contract Valued Item List.
    """

    resource_type = "ContractTermAssetValuedItem"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.effectiveTime = None
        """ Contract Valued Item Effective Tiem.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.entityCodeableConcept = None
        """ Contract Valued Item Type.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.entityReference = None
        """ Contract Valued Item Type.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.factor = None
        """ Contract Valued Item Price Scaling Factor.
        Type `float`. """

        self.identifier = None
        """ Contract Valued Item Number.
        Type `Identifier` (represented as `dict` in JSON). """

        self.linkId = None
        """ Pointer to specific item.
        List of `str` items. """

        self.net = None
        """ Total Contract Valued Item Value.
        Type `Money` (represented as `dict` in JSON). """

        self.payment = None
        """ Terms of valuation.
        Type `str`. """

        self.paymentDate = None
        """ When payment is due.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.points = None
        """ Contract Valued Item Difficulty Scaling Factor.
        Type `float`. """

        self.quantity = None
        """ Count of Contract Valued Items.
        Type `Quantity` (represented as `dict` in JSON). """

        self.recipient = None
        """ Who will receive payment.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.responsible = None
        """ Who will make payment.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.securityLabelNumber = None
        """ Security Labels that define affected terms.
        List of `int` items. """

        self.unitPrice = None
        """ Contract Valued Item fee, charge, or cost.
        Type `Money` (represented as `dict` in JSON). """

        super(ContractTermAssetValuedItem, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermAssetValuedItem, self).elementProperties()
        js.extend([
            ("effectiveTime", "effectiveTime", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("entityCodeableConcept", "entityCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "entity", False),
            ("entityReference", "entityReference", fhirreference.FHIRReference, "Reference", False, "entity", False),
            ("factor", "factor", float, "decimal", False, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", False, None, False),
            ("linkId", "linkId", str, "string", True, None, False),
            ("net", "net", money.Money, "Money", False, None, False),
            ("payment", "payment", str, "string", False, None, False),
            ("paymentDate", "paymentDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("points", "points", float, "decimal", False, None, False),
            ("quantity", "quantity", quantity.Quantity, "Quantity", False, None, False),
            ("recipient", "recipient", fhirreference.FHIRReference, "Reference", False, None, False),
            ("responsible", "responsible", fhirreference.FHIRReference, "Reference", False, None, False),
            ("securityLabelNumber", "securityLabelNumber", int, "unsignedInt", True, None, False),
            ("unitPrice", "unitPrice", money.Money, "Money", False, None, False),
        ])
        return js


class ContractTermOffer(backboneelement.BackboneElement):
    """ Context of the Contract term.

    The matter of concern in the context of this provision of the agrement.
    """

    resource_type = "ContractTermOffer"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.answer = None
        """ Response to offer text.
        List of `ContractTermOfferAnswer` items (represented as `dict` in JSON). """

        self.decision = None
        """ Accepting party choice.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.decisionMode = None
        """ How decision is conveyed.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Offer business ID.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.linkId = None
        """ Pointer to text.
        List of `str` items. """

        self.party = None
        """ Offer Recipient.
        List of `ContractTermOfferParty` items (represented as `dict` in JSON). """

        self.securityLabelNumber = None
        """ Offer restriction numbers.
        List of `int` items. """

        self.text = None
        """ Human readable offer text.
        Type `str`. """

        self.topic = None
        """ Negotiable offer asset.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.type = None
        """ Contract Offer Type or Form.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractTermOffer, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermOffer, self).elementProperties()
        js.extend([
            ("answer", "answer", ContractTermOfferAnswer, "ContractTermOfferAnswer", True, None, False),
            ("decision", "decision", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("decisionMode", "decisionMode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("linkId", "linkId", str, "string", True, None, False),
            ("party", "party", ContractTermOfferParty, "ContractTermOfferParty", True, None, False),
            ("securityLabelNumber", "securityLabelNumber", int, "unsignedInt", True, None, False),
            ("text", "text", str, "string", False, None, False),
            ("topic", "topic", fhirreference.FHIRReference, "Reference", False, None, False),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


class ContractTermOfferAnswer(backboneelement.BackboneElement):
    """ Response to offer text.
    """

    resource_type = "ContractTermOfferAnswer"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.valueAttachment = None
        """ The actual answer response.
        Type `Attachment` (represented as `dict` in JSON). """

        self.valueBoolean = None
        """ The actual answer response.
        Type `bool`. """

        self.valueCoding = None
        """ The actual answer response.
        Type `Coding` (represented as `dict` in JSON). """

        self.valueDate = None
        """ The actual answer response.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.valueDateTime = None
        """ The actual answer response.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.valueDecimal = None
        """ The actual answer response.
        Type `float`. """

        self.valueInteger = None
        """ The actual answer response.
        Type `int`. """

        self.valueQuantity = None
        """ The actual answer response.
        Type `Quantity` (represented as `dict` in JSON). """

        self.valueReference = None
        """ The actual answer response.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.valueString = None
        """ The actual answer response.
        Type `str`. """

        self.valueTime = None
        """ The actual answer response.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.valueUri = None
        """ The actual answer response.
        Type `str`. """

        super(ContractTermOfferAnswer, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermOfferAnswer, self).elementProperties()
        js.extend([
            ("valueAttachment", "valueAttachment", attachment.Attachment, "Attachment", False, "value", True),
            ("valueBoolean", "valueBoolean", bool, "boolean", False, "value", True),
            ("valueCoding", "valueCoding", coding.Coding, "Coding", False, "value", True),
            ("valueDate", "valueDate", fhirdate.FHIRDate, "date", False, "value", True),
            ("valueDateTime", "valueDateTime", fhirdate.FHIRDate, "dateTime", False, "value", True),
            ("valueDecimal", "valueDecimal", float, "decimal", False, "value", True),
            ("valueInteger", "valueInteger", int, "integer", False, "value", True),
            ("valueQuantity", "valueQuantity", quantity.Quantity, "Quantity", False, "value", True),
            ("valueReference", "valueReference", fhirreference.FHIRReference, "Reference", False, "value", True),
            ("valueString", "valueString", str, "string", False, "value", True),
            ("valueTime", "valueTime", fhirdate.FHIRDate, "time", False, "value", True),
            ("valueUri", "valueUri", str, "uri", False, "value", True),
        ])
        return js


class ContractTermOfferParty(backboneelement.BackboneElement):
    """ Offer Recipient.
    """

    resource_type = "ContractTermOfferParty"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.reference = None
        """ Referenced entity.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.role = None
        """ Participant engagement type.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ContractTermOfferParty, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermOfferParty, self).elementProperties()
        js.extend([
            ("reference", "reference", fhirreference.FHIRReference, "Reference", True, None, True),
            ("role", "role", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


class ContractTermSecurityLabel(backboneelement.BackboneElement):
    """ Protection for the Term.

    Security labels that protect the handling of information about the term and
    its elements, which may be specifically identified..
    """

    resource_type = "ContractTermSecurityLabel"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.category = None
        """ Applicable Policy.
        List of `Coding` items (represented as `dict` in JSON). """

        self.classification = None
        """ Confidentiality Protection.
        Type `Coding` (represented as `dict` in JSON). """

        self.control = None
        """ Handling Instructions.
        List of `Coding` items (represented as `dict` in JSON). """

        self.number = None
        """ Link to Security Labels.
        List of `int` items. """

        super(ContractTermSecurityLabel, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ContractTermSecurityLabel, self).elementProperties()
        js.extend([
            ("category", "category", coding.Coding, "Coding", True, None, False),
            ("classification", "classification", coding.Coding, "Coding", False, None, True),
            ("control", "control", coding.Coding, "Coding", True, None, False),
            ("number", "number", int, "unsignedInt", True, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import attachment
except ImportError:
    attachment = sys.modules[__package__ + '.attachment']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import money
except ImportError:
    money = sys.modules[__package__ + '.money']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import quantity
except ImportError:
    quantity = sys.modules[__package__ + '.quantity']
try:
    from . import signature
except ImportError:
    signature = sys.modules[__package__ + '.signature']
try:
    from . import timing
except ImportError:
    timing = sys.modules[__package__ + '.timing']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/ImagingStudy) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class ImagingStudy(domainresource.DomainResource):
    """ A set of images produced in single study (one or more series of references
    images).

    Representation of the content produced in a DICOM imaging study. A study
    comprises a set of series, each of which includes a set of Service-Object
    Pair Instances (SOP Instances - images or other data) acquired or produced
    in a common context.  A series is of only one modality (e.g. X-ray, CT, MR,
    ultrasound), but a study may have multiple series of different modalities.
    """

    resource_type = "ImagingStudy"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.basedOn = None
        """ Request fulfilled.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.description = None
        """ Institution-generated description.
        Type `str`. """

        self.encounter = None
        """ Encounter with which this imaging study is associated.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.endpoint = None
        """ Study access endpoint.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Identifiers for the whole study.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.interpreter = None
        """ Who interpreted images.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.location = None
        """ Where ImagingStudy occurred.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.modality = None
        """ All series modality if actual acquisition modalities.
        List of `Coding` items (represented as `dict` in JSON). """

        self.note = None
        """ User-defined comments.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.numberOfInstances = None
        """ Number of Study Related Instances.
        Type `int`. """

        self.numberOfSeries = None
        """ Number of Study Related Series.
        Type `int`. """

        self.procedureCode = None
        """ The performed procedure code.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.procedureReference = None
        """ The performed Procedure reference.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.reasonCode = None
        """ Why the study was requested.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.reasonReference = None
        """ Why was study performed.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.referrer = None
        """ Referring physician.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.series = None
        """ Each study has one or more series of instances.
        List of `ImagingStudySeries` items (represented as `dict` in JSON). """

        self.started = None
        """ When the study was started.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.status = None
        """ registered | available | cancelled | entered-in-error | unknown.
        Type `str`. """

        self.subject = None
        """ Who or what is the subject of the study.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(ImagingStudy, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ImagingStudy, self).elementProperties()
        js.extend([
            ("basedOn", "basedOn", fhirreference.FHIRReference, "Reference", True, None, False),
            ("description", "description", str, "string", False, None, False),
            ("encounter", "encounter", fhirreference.FHIRReference, "Reference", False, None, False),
            ("endpoint", "endpoint", fhirreference.FHIRReference, "Reference", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("interpreter", "interpreter", fhirreference.FHIRReference, "Reference", True, None, False),
            ("location", "location", fhirreference.FHIRReference, "Reference", False, None, False),
            ("modality", "modality", coding.Coding, "Coding", True, None, False),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("numberOfInstances", "numberOfInstances", int, "unsignedInt", False, None, False),
            ("numberOfSeries", "numberOfSeries", int, "unsignedInt", False, None, False),
            ("procedureCode", "procedureCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("procedureReference", "procedureReference", fhirreference.FHIRReference, "Reference", False, None, False),
            ("reasonCode", "reasonCode", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("reasonReference", "reasonReference", fhirreference.FHIRReference, "Reference", True, None, False),
            ("referrer", "referrer", fhirreference.FHIRReference, "Reference", False, None, False),
            ("series", "series", ImagingStudySeries, "ImagingStudySeries", True, None, False),
            ("started", "started", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("status", "status", str, "code", False, None, True),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", False, None, True),
        ])
        return js


from . import backboneelement

class ImagingStudySeries(backboneelement.BackboneElement):
    """ Each study has one or more series of instances.

    Each study has one or more series of images or other content.
    """

    resource_type = "ImagingStudySeries"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.bodySite = None
        """ Body part examined.
        Type `Coding` (represented as `dict` in JSON). """

        self.description = None
        """ A short human readable summary of the series.
        Type `str`. """

        self.endpoint = None
        """ Series access endpoint.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.instance = None
        """ A single SOP instance from the series.
        List of `ImagingStudySeriesInstance` items (represented as `dict` in JSON). """

        self.laterality = None
        """ Body part laterality.
        Type `Coding` (represented as `dict` in JSON). """

        self.modality = None
        """ The modality of the instances in the series.
        Type `Coding` (represented as `dict` in JSON). """

        self.number = None
        """ Numeric identifier of this series.
        Type `int`. """

        self.numberOfInstances = None
        """ Number of Series Related Instances.
        Type `int`. """

        self.performer = None
        """ Who performed the series.
        List of `ImagingStudySeriesPerformer` items (represented as `dict` in JSON). """

        self.specimen = None
        """ Specimen imaged.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.started = None
        """ When the series started.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.uid = None
        """ DICOM Series Instance UID for the series.
        Type `str`. """

        super(ImagingStudySeries, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ImagingStudySeries, self).elementProperties()
        js.extend([
            ("bodySite", "bodySite", coding.Coding, "Coding", False, None, False),
            ("description", "description", str, "string", False, None, False),
            ("endpoint", "endpoint", fhirreference.FHIRReference, "Reference", True, None, False),
            ("instance", "instance", ImagingStudySeriesInstance, "ImagingStudySeriesInstance", True, None, False),
            ("laterality", "laterality", coding.Coding, "Coding", False, None, False),
            ("modality", "modality", coding.Coding, "Coding", False, None, True),
            ("number", "number", int, "unsignedInt", False, None, False),
            ("numberOfInstances", "numberOfInstances", int, "unsignedInt", False, None, False),
            ("performer", "performer", ImagingStudySeriesPerformer, "ImagingStudySeriesPerformer", True, None, False),
            ("specimen", "specimen", fhirreference.FHIRReference, "Reference", True, None, False),
            ("started", "started", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("uid", "uid", str, "id", False, None, True),
        ])
        return js


class ImagingStudySeriesInstance(backboneelement.BackboneElement):
    """ A single SOP instance from the series.

    A single SOP instance within the series, e.g. an image, or presentation
    state.
    """

    resource_type = "ImagingStudySeriesInstance"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.number = None
        """ The number of this instance in the series.
        Type `int`. """

        self.sopClass = None
        """ DICOM class type.
        Type `Coding` (represented as `dict` in JSON). """

        self.title = None
        """ Description of instance.
        Type `str`. """

        self.uid = None
        """ DICOM SOP Instance UID.
        Type `str`. """

        super(ImagingStudySeriesInstance, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ImagingStudySeriesInstance, self).elementProperties()
        js.extend([
            ("number", "number", int, "unsignedInt", False, None, False),
            ("sopClass", "sopClass", coding.Coding, "Coding", False, None, True),
            ("title", "title", str, "string", False, None, False),
            ("uid", "uid", str, "id", False, None, True),
        ])
        return js


class ImagingStudySeriesPerformer(backboneelement.BackboneElement):
    """ Who performed the series.

    Indicates who or what performed the series and how they were involved.
    """

    resource_type = "ImagingStudySeriesPerformer"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.actor = None
        """ Who performed the series.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.function = None
        """ Type of performance.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(ImagingStudySeriesPerformer, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(ImagingStudySeriesPerformer, self).elementProperties()
        js.extend([
            ("actor", "actor", fhirreference.FHIRReference, "Reference", False, None, True),
            ("function", "function", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import structuredefinition
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class StructureDefinitionTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("StructureDefinition", js["resourceType"])
        return structuredefinition.StructureDefinition(js)
    
    def testStructureDefinition1(self):
        inst = self.instantiate_from("structuredefinition-example-section-library.json")
        self.assertIsNotNone(inst, "Must have instantiated a StructureDefinition instance")
        self.implStructureDefinition1(inst)
        
        js = inst.as_json()
        self.assertEqual("StructureDefinition", js["resourceType"])
        inst2 = structuredefinition.StructureDefinition(js)
        self.implStructureDefinition1(inst2)
    
    def implStructureDefinition1(self, inst):
        self.assertTrue(inst.abstract)
        self.assertEqual(force_bytes(inst.baseDefinition), force_bytes("http://hl7.org/fhir/StructureDefinition/Composition"))
        self.assertEqual(inst.date.date, FHIRDate("2018-11-05T17:57:00+11:00").date)
        self.assertEqual(inst.date.as_json(), "2018-11-05T17:57:00+11:00")
        self.assertEqual(force_bytes(inst.derivation), force_bytes("constraint"))
        self.assertEqual(force_bytes(inst.differential.element[0].id), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.differential.element[0].path), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.differential.element[1].id), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[1].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.description), force_bytes("Slice by .section.code when using this library of sections"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.discriminator[0].path), force_bytes("code"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.discriminator[0].type), force_bytes("pattern"))
        self.assertTrue(inst.differential.element[1].slicing.ordered)
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.rules), force_bytes("closed"))
        self.assertEqual(force_bytes(inst.differential.element[2].id), force_bytes("Composition.section:procedure"))
        self.assertEqual(force_bytes(inst.differential.element[2].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[2].sliceName), force_bytes("procedure"))
        self.assertEqual(force_bytes(inst.differential.element[3].fixedString), force_bytes("Procedures Performed"))
        self.assertEqual(force_bytes(inst.differential.element[3].id), force_bytes("Composition.section:procedure.title"))
        self.assertEqual(inst.differential.element[3].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[3].path), force_bytes("Composition.section.title"))
        self.assertEqual(force_bytes(inst.differential.element[4].id), force_bytes("Composition.section:procedure.code"))
        self.assertEqual(inst.differential.element[4].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[4].path), force_bytes("Composition.section.code"))
        self.assertEqual(force_bytes(inst.differential.element[4].patternCodeableConcept.coding[0].code), force_bytes("29554-3"))
        self.assertEqual(force_bytes(inst.differential.element[4].patternCodeableConcept.coding[0].display), force_bytes("Procedure Narrative"))
        self.assertEqual(force_bytes(inst.differential.element[4].patternCodeableConcept.coding[0].system), force_bytes("http://loinc.org"))
        self.assertEqual(force_bytes(inst.differential.element[5].id), force_bytes("Composition.section:medications"))
        self.assertEqual(force_bytes(inst.differential.element[5].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[5].sliceName), force_bytes("medications"))
        self.assertEqual(force_bytes(inst.differential.element[6].fixedString), force_bytes("Medications Administered"))
        self.assertEqual(force_bytes(inst.differential.element[6].id), force_bytes("Composition.section:medications.title"))
        self.assertEqual(inst.differential.element[6].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[6].path), force_bytes("Composition.section.title"))
        self.assertEqual(force_bytes(inst.differential.element[7].id), force_bytes("Composition.section:medications.code"))
        self.assertEqual(inst.differential.element[7].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[7].path), force_bytes("Composition.section.code"))
        self.assertEqual(force_bytes(inst.differential.element[7].patternCodeableConcept.coding[0].code), force_bytes("29549-3"))
        self.assertEqual(force_bytes(inst.differential.element[7].patternCodeableConcept.coding[0].display), force_bytes("Medication administered Narrative"))
        self.assertEqual(force_bytes(inst.differential.element[7].patternCodeableConcept.coding[0].system), force_bytes("http://loinc.org"))
        self.assertEqual(force_bytes(inst.differential.element[8].id), force_bytes("Composition.section:plan"))
        self.assertEqual(force_bytes(inst.differential.element[8].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[8].sliceName), force_bytes("plan"))
        self.assertEqual(force_bytes(inst.differential.element[9].fixedString), force_bytes("Discharge Treatment Plan"))
        self.assertEqual(force_bytes(inst.differential.element[9].id), force_bytes("Composition.section:plan.title"))
        self.assertEqual(inst.differential.element[9].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[9].path), force_bytes("Composition.section.title"))
        self.assertFalse(inst.experimental)
        self.assertEqual(force_bytes(inst.id), force_bytes("example-section-library"))
        self.assertEqual(force_bytes(inst.kind), force_bytes("complex-type"))
        self.assertEqual(force_bytes(inst.name), force_bytes("DocumentSectionLibrary"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.title), force_bytes("Document Section Library (For testing section templates)"))
        self.assertEqual(force_bytes(inst.type), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.url), force_bytes("http://hl7.org/fhir/StructureDefinition/example-section-library"))
    
    def testStructureDefinition2(self):
        inst = self.instantiate_from("structuredefinition-example-composition.json")
        self.assertIsNotNone(inst, "Must have instantiated a StructureDefinition instance")
        self.implStructureDefinition2(inst)
        
        js = inst.as_json()
        self.assertEqual("StructureDefinition", js["resourceType"])
        inst2 = structuredefinition.StructureDefinition(js)
        self.implStructureDefinition2(inst2)
    
    def implStructureDefinition2(self, inst):
        self.assertFalse(inst.abstract)
        self.assertEqual(force_bytes(inst.baseDefinition), force_bytes("http://hl7.org/fhir/StructureDefinition/Composition"))
        self.assertEqual(inst.date.date, FHIRDate("2018-11-05T17:47:00+11:00").date)
        self.assertEqual(inst.date.as_json(), "2018-11-05T17:47:00+11:00")
        self.assertEqual(force_bytes(inst.derivation), force_bytes("constraint"))
        self.assertEqual(force_bytes(inst.differential.element[0].id), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.differential.element[0].path), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.differential.element[1].id), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[1].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.description), force_bytes("Slice by .section.code"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.discriminator[0].path), force_bytes("code"))
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.discriminator[0].type), force_bytes("pattern"))
        self.assertTrue(inst.differential.element[1].slicing.ordered)
        self.assertEqual(force_bytes(inst.differential.element[1].slicing.rules), force_bytes("closed"))
        self.assertEqual(force_bytes(inst.differential.element[2].id), force_bytes("Composition.section:procedure"))
        self.assertEqual(inst.differential.element[2].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[2].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[2].sliceName), force_bytes("procedure"))
        self.assertEqual(force_bytes(inst.differential.element[2].type[0].code), force_bytes("BackboneElement"))
        self.assertEqual(force_bytes(inst.differential.element[2].type[0].profile[0]), force_bytes("http://hl7.org/fhir/StructureDefinition/document-section-library"))
        self.assertEqual(force_bytes(inst.differential.element[3].id), force_bytes("Composition.section:medications"))
        self.assertEqual(inst.differential.element[3].min, 1)
        self.assertEqual(force_bytes(inst.differential.element[3].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[3].sliceName), force_bytes("medications"))
        self.assertEqual(force_bytes(inst.differential.element[3].type[0].code), force_bytes("BackboneElement"))
        self.assertEqual(force_bytes(inst.differential.element[3].type[0].profile[0]), force_bytes("http://hl7.org/fhir/StructureDefinition/document-section-library"))
        self.assertEqual(force_bytes(inst.differential.element[4].id), force_bytes("Composition.section:plan"))
        self.assertEqual(inst.differential.element[4].min, 0)
        self.assertEqual(force_bytes(inst.differential.element[4].path), force_bytes("Composition.section"))
        self.assertEqual(force_bytes(inst.differential.element[4].sliceName), force_bytes("plan"))
        self.assertEqual(force_bytes(inst.differential.element[4].type[0].code), force_bytes("BackboneElement"))
        self.assertEqual(force_bytes(inst.differential.element[4].type[0].profile[0]), force_bytes("http://hl7.org/fhir/StructureDefinition/document-section-library"))
        self.assertFalse(inst.experimental)
        self.assertEqual(force_bytes(inst.id), force_bytes("example-composition"))
        self.assertEqual(force_bytes(inst.kind), force_bytes("complex-type"))
        self.assertEqual(force_bytes(inst.name), force_bytes("DocumentStructure"))
        self.assertEqual(force_bytes(inst.status), force_bytes("active"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
        self.assertEqual(force_bytes(inst.title), force_bytes("Document Structure (For testing section templates)"))
        self.assertEqual(force_bytes(inst.type), force_bytes("Composition"))
        self.assertEqual(force_bytes(inst.url), force_bytes("http://hl7.org/fhir/StructureDefinition/example-composition"))




"""FHIR (version 1.0.2.7202) DSTU2 resources.
This whole package is generated using
https://github.com/smart-on-fhir/fhir-parser/tree/b9140c8a189b9cf7efe882760aa55ad88dc361ef

All credit: SMART Health IT.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Population) on 2019-05-13.
#  2019, SMART Health IT.


from . import backboneelement

class Population(backboneelement.BackboneElement):
    """ A definition of a set of people that apply to some clinically related
    context, for example people contraindicated for a certain medication.

    A populatioof people with some set of grouping criteria.
    """

    resource_type = "Population"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.ageCodeableConcept = None
        """ The age of the specific population.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.ageRange = None
        """ The age of the specific population.
        Type `Range` (represented as `dict` in JSON). """

        self.gender = None
        """ The gender of the specific population.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.physiologicalCondition = None
        """ The existing physiological conditions of the specific population to
        which this applies.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.race = None
        """ Race of the specific population.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(Population, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Population, self).elementProperties()
        js.extend([
            ("ageCodeableConcept", "ageCodeableConcept", codeableconcept.CodeableConcept, "CodeableConcept", False, "age", False),
            ("ageRange", "ageRange", range.Range, "Range", False, "age", False),
            ("gender", "gender", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("physiologicalCondition", "physiologicalCondition", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("race", "race", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import range
except ImportError:
    range = sys.modules[__package__ + '.range']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Evidence) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Evidence(domainresource.DomainResource):
    """ A research context or question.

    The Evidence resource describes the conditional state (population and any
    exposures being compared within the population) and outcome (if specified)
    that the knowledge (evidence, assertion, recommendation) is about.
    """

    resource_type = "Evidence"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.approvalDate = None
        """ When the evidence was approved by publisher.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.author = None
        """ Who authored the content.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.copyright = None
        """ Use and/or publishing restrictions.
        Type `str`. """

        self.date = None
        """ Date last changed.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.description = None
        """ Natural language description of the evidence.
        Type `str`. """

        self.editor = None
        """ Who edited the content.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.effectivePeriod = None
        """ When the evidence is expected to be used.
        Type `Period` (represented as `dict` in JSON). """

        self.endorser = None
        """ Who endorsed the content.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.exposureBackground = None
        """ What population?.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.exposureVariant = None
        """ What exposure?.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Additional identifier for the evidence.
        List of `Identifier` items (represented as `dict` in JSON). """

        self.jurisdiction = None
        """ Intended jurisdiction for evidence (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.lastReviewDate = None
        """ When the evidence was last reviewed.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.name = None
        """ Name for this evidence (computer friendly).
        Type `str`. """

        self.note = None
        """ Used for footnotes or explanatory notes.
        List of `Annotation` items (represented as `dict` in JSON). """

        self.outcome = None
        """ What outcome?.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """

        self.relatedArtifact = None
        """ Additional documentation, citations, etc..
        List of `RelatedArtifact` items (represented as `dict` in JSON). """

        self.reviewer = None
        """ Who reviewed the content.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.shortTitle = None
        """ Title for use in informal contexts.
        Type `str`. """

        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """

        self.subtitle = None
        """ Subordinate title of the Evidence.
        Type `str`. """

        self.title = None
        """ Name for this evidence (human friendly).
        Type `str`. """

        self.topic = None
        """ The category of the Evidence, such as Education, Treatment,
        Assessment, etc..
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.url = None
        """ Canonical identifier for this evidence, represented as a URI
        (globally unique).
        Type `str`. """

        self.useContext = None
        """ The context that the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """

        self.version = None
        """ Business version of the evidence.
        Type `str`. """

        super(Evidence, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Evidence, self).elementProperties()
        js.extend([
            ("approvalDate", "approvalDate", fhirdate.FHIRDate, "date", False, None, False),
            ("author", "author", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("contact", "contact", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("copyright", "copyright", str, "markdown", False, None, False),
            ("date", "date", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("description", "description", str, "markdown", False, None, False),
            ("editor", "editor", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("effectivePeriod", "effectivePeriod", period.Period, "Period", False, None, False),
            ("endorser", "endorser", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("exposureBackground", "exposureBackground", fhirreference.FHIRReference, "Reference", False, None, True),
            ("exposureVariant", "exposureVariant", fhirreference.FHIRReference, "Reference", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", True, None, False),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("lastReviewDate", "lastReviewDate", fhirdate.FHIRDate, "date", False, None, False),
            ("name", "name", str, "string", False, None, False),
            ("note", "note", annotation.Annotation, "Annotation", True, None, False),
            ("outcome", "outcome", fhirreference.FHIRReference, "Reference", True, None, False),
            ("publisher", "publisher", str, "string", False, None, False),
            ("relatedArtifact", "relatedArtifact", relatedartifact.RelatedArtifact, "RelatedArtifact", True, None, False),
            ("reviewer", "reviewer", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("shortTitle", "shortTitle", str, "string", False, None, False),
            ("status", "status", str, "code", False, None, True),
            ("subtitle", "subtitle", str, "string", False, None, False),
            ("title", "title", str, "string", False, None, False),
            ("topic", "topic", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("url", "url", str, "uri", False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, "UsageContext", True, None, False),
            ("version", "version", str, "string", False, None, False),
        ])
        return js


import sys
try:
    from . import annotation
except ImportError:
    annotation = sys.modules[__package__ + '.annotation']
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']
try:
    from . import relatedartifact
except ImportError:
    relatedartifact = sys.modules[__package__ + '.relatedartifact']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/CapabilityStatement) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class CapabilityStatement(domainresource.DomainResource):
    """ A statement of system capabilities.

    A Capability Statement documents a set of capabilities (behaviors) of a
    FHIR Server for a particular version of FHIR that may be used as a
    statement of actual server functionality or a statement of required or
    desired server implementation.
    """

    resource_type = "CapabilityStatement"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.contact = None
        """ Contact details for the publisher.
        List of `ContactDetail` items (represented as `dict` in JSON). """

        self.copyright = None
        """ Use and/or publishing restrictions.
        Type `str`. """

        self.date = None
        """ Date last changed.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.description = None
        """ Natural language description of the capability statement.
        Type `str`. """

        self.document = None
        """ Document definition.
        List of `CapabilityStatementDocument` items (represented as `dict` in JSON). """

        self.experimental = None
        """ For testing purposes, not real usage.
        Type `bool`. """

        self.fhirVersion = None
        """ FHIR Version the system supports.
        Type `str`. """

        self.format = None
        """ formats supported (xml | json | ttl | mime type).
        List of `str` items. """

        self.implementation = None
        """ If this describes a specific instance.
        Type `CapabilityStatementImplementation` (represented as `dict` in JSON). """

        self.implementationGuide = None
        """ Implementation guides supported.
        List of `str` items. """

        self.imports = None
        """ Canonical URL of another capability statement this adds to.
        List of `str` items. """

        self.instantiates = None
        """ Canonical URL of another capability statement this implements.
        List of `str` items. """

        self.jurisdiction = None
        """ Intended jurisdiction for capability statement (if applicable).
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.kind = None
        """ instance | capability | requirements.
        Type `str`. """

        self.messaging = None
        """ If messaging is supported.
        List of `CapabilityStatementMessaging` items (represented as `dict` in JSON). """

        self.name = None
        """ Name for this capability statement (computer friendly).
        Type `str`. """

        self.patchFormat = None
        """ Patch formats supported.
        List of `str` items. """

        self.publisher = None
        """ Name of the publisher (organization or individual).
        Type `str`. """

        self.purpose = None
        """ Why this capability statement is defined.
        Type `str`. """

        self.rest = None
        """ If the endpoint is a RESTful one.
        List of `CapabilityStatementRest` items (represented as `dict` in JSON). """

        self.software = None
        """ Software that is covered by this capability statement.
        Type `CapabilityStatementSoftware` (represented as `dict` in JSON). """

        self.status = None
        """ draft | active | retired | unknown.
        Type `str`. """

        self.title = None
        """ Name for this capability statement (human friendly).
        Type `str`. """

        self.url = None
        """ Canonical identifier for this capability statement, represented as
        a URI (globally unique).
        Type `str`. """

        self.useContext = None
        """ The context that the content is intended to support.
        List of `UsageContext` items (represented as `dict` in JSON). """

        self.version = None
        """ Business version of the capability statement.
        Type `str`. """

        super(CapabilityStatement, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatement, self).elementProperties()
        js.extend([
            ("contact", "contact", contactdetail.ContactDetail, "ContactDetail", True, None, False),
            ("copyright", "copyright", str, "markdown", False, None, False),
            ("date", "date", fhirdate.FHIRDate, "dateTime", False, None, True),
            ("description", "description", str, "markdown", False, None, False),
            ("document", "document", CapabilityStatementDocument, "CapabilityStatementDocument", True, None, False),
            ("experimental", "experimental", bool, "boolean", False, None, False),
            ("fhirVersion", "fhirVersion", str, "code", False, None, True),
            ("format", "format", str, "code", True, None, True),
            ("implementation", "implementation", CapabilityStatementImplementation, "CapabilityStatementImplementation", False, None, False),
            ("implementationGuide", "implementationGuide", str, "canonical", True, None, False),
            ("imports", "imports", str, "canonical", True, None, False),
            ("instantiates", "instantiates", str, "canonical", True, None, False),
            ("jurisdiction", "jurisdiction", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("kind", "kind", str, "code", False, None, True),
            ("messaging", "messaging", CapabilityStatementMessaging, "CapabilityStatementMessaging", True, None, False),
            ("name", "name", str, "string", False, None, False),
            ("patchFormat", "patchFormat", str, "code", True, None, False),
            ("publisher", "publisher", str, "string", False, None, False),
            ("purpose", "purpose", str, "markdown", False, None, False),
            ("rest", "rest", CapabilityStatementRest, "CapabilityStatementRest", True, None, False),
            ("software", "software", CapabilityStatementSoftware, "CapabilityStatementSoftware", False, None, False),
            ("status", "status", str, "code", False, None, True),
            ("title", "title", str, "string", False, None, False),
            ("url", "url", str, "uri", False, None, False),
            ("useContext", "useContext", usagecontext.UsageContext, "UsageContext", True, None, False),
            ("version", "version", str, "string", False, None, False),
        ])
        return js


from . import backboneelement

class CapabilityStatementDocument(backboneelement.BackboneElement):
    """ Document definition.

    A document definition.
    """

    resource_type = "CapabilityStatementDocument"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.documentation = None
        """ Description of document support.
        Type `str`. """

        self.mode = None
        """ producer | consumer.
        Type `str`. """

        self.profile = None
        """ Constraint on the resources used in the document.
        Type `str`. """

        super(CapabilityStatementDocument, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementDocument, self).elementProperties()
        js.extend([
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("mode", "mode", str, "code", False, None, True),
            ("profile", "profile", str, "canonical", False, None, True),
        ])
        return js


class CapabilityStatementImplementation(backboneelement.BackboneElement):
    """ If this describes a specific instance.

    Identifies a specific implementation instance that is described by the
    capability statement - i.e. a particular installation, rather than the
    capabilities of a software program.
    """

    resource_type = "CapabilityStatementImplementation"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.custodian = None
        """ Organization that manages the data.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.description = None
        """ Describes this specific instance.
        Type `str`. """

        self.url = None
        """ Base URL for the installation.
        Type `str`. """

        super(CapabilityStatementImplementation, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementImplementation, self).elementProperties()
        js.extend([
            ("custodian", "custodian", fhirreference.FHIRReference, "Reference", False, None, False),
            ("description", "description", str, "string", False, None, True),
            ("url", "url", str, "url", False, None, False),
        ])
        return js


class CapabilityStatementMessaging(backboneelement.BackboneElement):
    """ If messaging is supported.

    A description of the messaging capabilities of the solution.
    """

    resource_type = "CapabilityStatementMessaging"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.documentation = None
        """ Messaging interface behavior details.
        Type `str`. """

        self.endpoint = None
        """ Where messages should be sent.
        List of `CapabilityStatementMessagingEndpoint` items (represented as `dict` in JSON). """

        self.reliableCache = None
        """ Reliable Message Cache Length (min).
        Type `int`. """

        self.supportedMessage = None
        """ Messages supported by this system.
        List of `CapabilityStatementMessagingSupportedMessage` items (represented as `dict` in JSON). """

        super(CapabilityStatementMessaging, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementMessaging, self).elementProperties()
        js.extend([
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("endpoint", "endpoint", CapabilityStatementMessagingEndpoint, "CapabilityStatementMessagingEndpoint", True, None, False),
            ("reliableCache", "reliableCache", int, "unsignedInt", False, None, False),
            ("supportedMessage", "supportedMessage", CapabilityStatementMessagingSupportedMessage, "CapabilityStatementMessagingSupportedMessage", True, None, False),
        ])
        return js


class CapabilityStatementMessagingEndpoint(backboneelement.BackboneElement):
    """ Where messages should be sent.

    An endpoint (network accessible address) to which messages and/or replies
    are to be sent.
    """

    resource_type = "CapabilityStatementMessagingEndpoint"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.address = None
        """ Network address or identifier of the end-point.
        Type `str`. """

        self.protocol = None
        """ http | ftp | mllp +.
        Type `Coding` (represented as `dict` in JSON). """

        super(CapabilityStatementMessagingEndpoint, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementMessagingEndpoint, self).elementProperties()
        js.extend([
            ("address", "address", str, "url", False, None, True),
            ("protocol", "protocol", coding.Coding, "Coding", False, None, True),
        ])
        return js


class CapabilityStatementMessagingSupportedMessage(backboneelement.BackboneElement):
    """ Messages supported by this system.

    References to message definitions for messages this system can send or
    receive.
    """

    resource_type = "CapabilityStatementMessagingSupportedMessage"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.definition = None
        """ Message supported by this system.
        Type `str`. """

        self.mode = None
        """ sender | receiver.
        Type `str`. """

        super(CapabilityStatementMessagingSupportedMessage, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementMessagingSupportedMessage, self).elementProperties()
        js.extend([
            ("definition", "definition", str, "canonical", False, None, True),
            ("mode", "mode", str, "code", False, None, True),
        ])
        return js


class CapabilityStatementRest(backboneelement.BackboneElement):
    """ If the endpoint is a RESTful one.

    A definition of the restful capabilities of the solution, if any.
    """

    resource_type = "CapabilityStatementRest"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.compartment = None
        """ Compartments served/used by system.
        List of `str` items. """

        self.documentation = None
        """ General description of implementation.
        Type `str`. """

        self.interaction = None
        """ What operations are supported?.
        List of `CapabilityStatementRestInteraction` items (represented as `dict` in JSON). """

        self.mode = None
        """ client | server.
        Type `str`. """

        self.operation = None
        """ Definition of a system level operation.
        List of `CapabilityStatementRestResourceOperation` items (represented as `dict` in JSON). """

        self.resource = None
        """ Resource served on the REST interface.
        List of `CapabilityStatementRestResource` items (represented as `dict` in JSON). """

        self.searchParam = None
        """ Search parameters for searching all resources.
        List of `CapabilityStatementRestResourceSearchParam` items (represented as `dict` in JSON). """

        self.security = None
        """ Information about security of implementation.
        Type `CapabilityStatementRestSecurity` (represented as `dict` in JSON). """

        super(CapabilityStatementRest, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRest, self).elementProperties()
        js.extend([
            ("compartment", "compartment", str, "canonical", True, None, False),
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("interaction", "interaction", CapabilityStatementRestInteraction, "CapabilityStatementRestInteraction", True, None, False),
            ("mode", "mode", str, "code", False, None, True),
            ("operation", "operation", CapabilityStatementRestResourceOperation, "CapabilityStatementRestResourceOperation", True, None, False),
            ("resource", "resource", CapabilityStatementRestResource, "CapabilityStatementRestResource", True, None, False),
            ("searchParam", "searchParam", CapabilityStatementRestResourceSearchParam, "CapabilityStatementRestResourceSearchParam", True, None, False),
            ("security", "security", CapabilityStatementRestSecurity, "CapabilityStatementRestSecurity", False, None, False),
        ])
        return js


class CapabilityStatementRestInteraction(backboneelement.BackboneElement):
    """ What operations are supported?.

    A specification of restful operations supported by the system.
    """

    resource_type = "CapabilityStatementRestInteraction"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ transaction | batch | search-system | history-system.
        Type `str`. """

        self.documentation = None
        """ Anything special about operation behavior.
        Type `str`. """

        super(CapabilityStatementRestInteraction, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestInteraction, self).elementProperties()
        js.extend([
            ("code", "code", str, "code", False, None, True),
            ("documentation", "documentation", str, "markdown", False, None, False),
        ])
        return js


class CapabilityStatementRestResource(backboneelement.BackboneElement):
    """ Resource served on the REST interface.

    A specification of the restful capabilities of the solution for a specific
    resource type.
    """

    resource_type = "CapabilityStatementRestResource"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.conditionalCreate = None
        """ If allows/uses conditional create.
        Type `bool`. """

        self.conditionalDelete = None
        """ not-supported | single | multiple - how conditional delete is
        supported.
        Type `str`. """

        self.conditionalRead = None
        """ not-supported | modified-since | not-match | full-support.
        Type `str`. """

        self.conditionalUpdate = None
        """ If allows/uses conditional update.
        Type `bool`. """

        self.documentation = None
        """ Additional information about the use of the resource type.
        Type `str`. """

        self.interaction = None
        """ What operations are supported?.
        List of `CapabilityStatementRestResourceInteraction` items (represented as `dict` in JSON). """

        self.operation = None
        """ Definition of a resource operation.
        List of `CapabilityStatementRestResourceOperation` items (represented as `dict` in JSON). """

        self.profile = None
        """ Base System profile for all uses of resource.
        Type `str`. """

        self.readHistory = None
        """ Whether vRead can return past versions.
        Type `bool`. """

        self.referencePolicy = None
        """ literal | logical | resolves | enforced | local.
        List of `str` items. """

        self.searchInclude = None
        """ _include values supported by the server.
        List of `str` items. """

        self.searchParam = None
        """ Search parameters supported by implementation.
        List of `CapabilityStatementRestResourceSearchParam` items (represented as `dict` in JSON). """

        self.searchRevInclude = None
        """ _revinclude values supported by the server.
        List of `str` items. """

        self.supportedProfile = None
        """ Profiles for use cases supported.
        List of `str` items. """

        self.type = None
        """ A resource type that is supported.
        Type `str`. """

        self.updateCreate = None
        """ If update can commit to a new identity.
        Type `bool`. """

        self.versioning = None
        """ no-version | versioned | versioned-update.
        Type `str`. """

        super(CapabilityStatementRestResource, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestResource, self).elementProperties()
        js.extend([
            ("conditionalCreate", "conditionalCreate", bool, "boolean", False, None, False),
            ("conditionalDelete", "conditionalDelete", str, "code", False, None, False),
            ("conditionalRead", "conditionalRead", str, "code", False, None, False),
            ("conditionalUpdate", "conditionalUpdate", bool, "boolean", False, None, False),
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("interaction", "interaction", CapabilityStatementRestResourceInteraction, "CapabilityStatementRestResourceInteraction", True, None, False),
            ("operation", "operation", CapabilityStatementRestResourceOperation, "CapabilityStatementRestResourceOperation", True, None, False),
            ("profile", "profile", str, "canonical", False, None, False),
            ("readHistory", "readHistory", bool, "boolean", False, None, False),
            ("referencePolicy", "referencePolicy", str, "code", True, None, False),
            ("searchInclude", "searchInclude", str, "string", True, None, False),
            ("searchParam", "searchParam", CapabilityStatementRestResourceSearchParam, "CapabilityStatementRestResourceSearchParam", True, None, False),
            ("searchRevInclude", "searchRevInclude", str, "string", True, None, False),
            ("supportedProfile", "supportedProfile", str, "canonical", True, None, False),
            ("type", "type", str, "code", False, None, True),
            ("updateCreate", "updateCreate", bool, "boolean", False, None, False),
            ("versioning", "versioning", str, "code", False, None, False),
        ])
        return js


class CapabilityStatementRestResourceInteraction(backboneelement.BackboneElement):
    """ What operations are supported?.

    Identifies a restful operation supported by the solution.
    """

    resource_type = "CapabilityStatementRestResourceInteraction"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ read | vread | update | patch | delete | history-instance |
        history-type | create | search-type.
        Type `str`. """

        self.documentation = None
        """ Anything special about operation behavior.
        Type `str`. """

        super(CapabilityStatementRestResourceInteraction, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestResourceInteraction, self).elementProperties()
        js.extend([
            ("code", "code", str, "code", False, None, True),
            ("documentation", "documentation", str, "markdown", False, None, False),
        ])
        return js


class CapabilityStatementRestResourceOperation(backboneelement.BackboneElement):
    """ Definition of a resource operation.

    Definition of an operation or a named query together with its parameters
    and their meaning and type. Consult the definition of the operation for
    details about how to invoke the operation, and the parameters.
    """

    resource_type = "CapabilityStatementRestResourceOperation"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.definition = None
        """ The defined operation/query.
        Type `str`. """

        self.documentation = None
        """ Specific details about operation behavior.
        Type `str`. """

        self.name = None
        """ Name by which the operation/query is invoked.
        Type `str`. """

        super(CapabilityStatementRestResourceOperation, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestResourceOperation, self).elementProperties()
        js.extend([
            ("definition", "definition", str, "canonical", False, None, True),
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("name", "name", str, "string", False, None, True),
        ])
        return js


class CapabilityStatementRestResourceSearchParam(backboneelement.BackboneElement):
    """ Search parameters supported by implementation.

    Search parameters for implementations to support and/or make use of -
    either references to ones defined in the specification, or additional ones
    defined for/by the implementation.
    """

    resource_type = "CapabilityStatementRestResourceSearchParam"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.definition = None
        """ Source of definition for parameter.
        Type `str`. """

        self.documentation = None
        """ Server-specific usage.
        Type `str`. """

        self.name = None
        """ Name of search parameter.
        Type `str`. """

        self.type = None
        """ number | date | string | token | reference | composite | quantity |
        uri | special.
        Type `str`. """

        super(CapabilityStatementRestResourceSearchParam, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestResourceSearchParam, self).elementProperties()
        js.extend([
            ("definition", "definition", str, "canonical", False, None, False),
            ("documentation", "documentation", str, "markdown", False, None, False),
            ("name", "name", str, "string", False, None, True),
            ("type", "type", str, "code", False, None, True),
        ])
        return js


class CapabilityStatementRestSecurity(backboneelement.BackboneElement):
    """ Information about security of implementation.

    Information about security implementation from an interface perspective -
    what a client needs to know.
    """

    resource_type = "CapabilityStatementRestSecurity"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.cors = None
        """ Adds CORS Headers (http://enable-cors.org/).
        Type `bool`. """

        self.description = None
        """ General description of how security works.
        Type `str`. """

        self.service = None
        """ OAuth | SMART-on-FHIR | NTLM | Basic | Kerberos | Certificates.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        super(CapabilityStatementRestSecurity, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementRestSecurity, self).elementProperties()
        js.extend([
            ("cors", "cors", bool, "boolean", False, None, False),
            ("description", "description", str, "markdown", False, None, False),
            ("service", "service", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
        ])
        return js


class CapabilityStatementSoftware(backboneelement.BackboneElement):
    """ Software that is covered by this capability statement.

    Software that is covered by this capability statement.  It is used when the
    capability statement describes the capabilities of a particular software
    version, independent of an installation.
    """

    resource_type = "CapabilityStatementSoftware"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.name = None
        """ A name the software is known by.
        Type `str`. """

        self.releaseDate = None
        """ Date this version was released.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.version = None
        """ Version covered by this statement.
        Type `str`. """

        super(CapabilityStatementSoftware, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CapabilityStatementSoftware, self).elementProperties()
        js.extend([
            ("name", "name", str, "string", False, None, True),
            ("releaseDate", "releaseDate", fhirdate.FHIRDate, "dateTime", False, None, False),
            ("version", "version", str, "string", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import coding
except ImportError:
    coding = sys.modules[__package__ + '.coding']
try:
    from . import contactdetail
except ImportError:
    contactdetail = sys.modules[__package__ + '.contactdetail']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import usagecontext
except ImportError:
    usagecontext = sys.modules[__package__ + '.usagecontext']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 4.0.0-a53ec6ee1b (http://hl7.org/fhir/StructureDefinition/Composition) on 2019-05-13.
#  2019, SMART Health IT.


from . import domainresource

class Composition(domainresource.DomainResource):
    """ A set of resources composed into a single coherent clinical statement with
    clinical attestation.

    A set of healthcare-related information that is assembled together into a
    single logical package that provides a single coherent statement of
    meaning, establishes its own context and that has clinical attestation with
    regard to who is making the statement. A Composition defines the structure
    and narrative content necessary for a document. However, a Composition
    alone does not constitute a document. Rather, the Composition must be the
    first entry in a Bundle where Bundle.type=document, and any other resources
    referenced from Composition must be included as subsequent entries in the
    Bundle (for example Patient, Practitioner, Encounter, etc.).
    """

    resource_type = "Composition"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.attester = None
        """ Attests to accuracy of composition.
        List of `CompositionAttester` items (represented as `dict` in JSON). """

        self.author = None
        """ Who and/or what authored the composition.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.category = None
        """ Categorization of Composition.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.confidentiality = None
        """ As defined by affinity domain.
        Type `str`. """

        self.custodian = None
        """ Organization which maintains the composition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.date = None
        """ Composition editing time.
        Type `FHIRDate` (represented as `str` in JSON). """

        self.encounter = None
        """ Context of the Composition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.event = None
        """ The clinical service(s) being documented.
        List of `CompositionEvent` items (represented as `dict` in JSON). """

        self.identifier = None
        """ Version-independent identifier for the Composition.
        Type `Identifier` (represented as `dict` in JSON). """

        self.relatesTo = None
        """ Relationships to other compositions/documents.
        List of `CompositionRelatesTo` items (represented as `dict` in JSON). """

        self.section = None
        """ Composition is broken into sections.
        List of `CompositionSection` items (represented as `dict` in JSON). """

        self.status = None
        """ preliminary | final | amended | entered-in-error.
        Type `str`. """

        self.subject = None
        """ Who and/or what the composition is about.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.title = None
        """ Human Readable name/title.
        Type `str`. """

        self.type = None
        """ Kind of composition (LOINC if possible).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        super(Composition, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(Composition, self).elementProperties()
        js.extend([
            ("attester", "attester", CompositionAttester, "CompositionAttester", True, None, False),
            ("author", "author", fhirreference.FHIRReference, "Reference", True, None, True),
            ("category", "category", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("confidentiality", "confidentiality", str, "code", False, None, False),
            ("custodian", "custodian", fhirreference.FHIRReference, "Reference", False, None, False),
            ("date", "date", fhirdate.FHIRDate, "dateTime", False, None, True),
            ("encounter", "encounter", fhirreference.FHIRReference, "Reference", False, None, False),
            ("event", "event", CompositionEvent, "CompositionEvent", True, None, False),
            ("identifier", "identifier", identifier.Identifier, "Identifier", False, None, False),
            ("relatesTo", "relatesTo", CompositionRelatesTo, "CompositionRelatesTo", True, None, False),
            ("section", "section", CompositionSection, "CompositionSection", True, None, False),
            ("status", "status", str, "code", False, None, True),
            ("subject", "subject", fhirreference.FHIRReference, "Reference", False, None, False),
            ("title", "title", str, "string", False, None, True),
            ("type", "type", codeableconcept.CodeableConcept, "CodeableConcept", False, None, True),
        ])
        return js


from . import backboneelement

class CompositionAttester(backboneelement.BackboneElement):
    """ Attests to accuracy of composition.

    A participant who has attested to the accuracy of the composition/document.
    """

    resource_type = "CompositionAttester"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.mode = None
        """ personal | professional | legal | official.
        Type `str`. """

        self.party = None
        """ Who attested the composition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.time = None
        """ When the composition was attested.
        Type `FHIRDate` (represented as `str` in JSON). """

        super(CompositionAttester, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CompositionAttester, self).elementProperties()
        js.extend([
            ("mode", "mode", str, "code", False, None, True),
            ("party", "party", fhirreference.FHIRReference, "Reference", False, None, False),
            ("time", "time", fhirdate.FHIRDate, "dateTime", False, None, False),
        ])
        return js


class CompositionEvent(backboneelement.BackboneElement):
    """ The clinical service(s) being documented.

    The clinical service, such as a colonoscopy or an appendectomy, being
    documented.
    """

    resource_type = "CompositionEvent"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ Code(s) that apply to the event being documented.
        List of `CodeableConcept` items (represented as `dict` in JSON). """

        self.detail = None
        """ The event(s) being documented.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.period = None
        """ The period covered by the documentation.
        Type `Period` (represented as `dict` in JSON). """

        super(CompositionEvent, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CompositionEvent, self).elementProperties()
        js.extend([
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", True, None, False),
            ("detail", "detail", fhirreference.FHIRReference, "Reference", True, None, False),
            ("period", "period", period.Period, "Period", False, None, False),
        ])
        return js


class CompositionRelatesTo(backboneelement.BackboneElement):
    """ Relationships to other compositions/documents.

    Relationships that this composition has with other compositions or
    documents that already exist.
    """

    resource_type = "CompositionRelatesTo"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.code = None
        """ replaces | transforms | signs | appends.
        Type `str`. """

        self.targetIdentifier = None
        """ Target of the relationship.
        Type `Identifier` (represented as `dict` in JSON). """

        self.targetReference = None
        """ Target of the relationship.
        Type `FHIRReference` (represented as `dict` in JSON). """

        super(CompositionRelatesTo, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CompositionRelatesTo, self).elementProperties()
        js.extend([
            ("code", "code", str, "code", False, None, True),
            ("targetIdentifier", "targetIdentifier", identifier.Identifier, "Identifier", False, "target", True),
            ("targetReference", "targetReference", fhirreference.FHIRReference, "Reference", False, "target", True),
        ])
        return js


class CompositionSection(backboneelement.BackboneElement):
    """ Composition is broken into sections.

    The root of the sections that make up the composition.
    """

    resource_type = "CompositionSection"

    def __init__(self, jsondict=None, strict=True):
        """ Initialize all valid properties.

        :raises: FHIRValidationError on validation errors, unless strict is False
        :param dict jsondict: A JSON dictionary to use for initialization
        :param bool strict: If True (the default), invalid variables will raise a TypeError
        """

        self.author = None
        """ Who and/or what authored the section.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.code = None
        """ Classification of section (recommended).
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.emptyReason = None
        """ Why the section is empty.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.entry = None
        """ A reference to data that supports this section.
        List of `FHIRReference` items (represented as `dict` in JSON). """

        self.focus = None
        """ Who/what the section is about, when it is not about the subject of
        composition.
        Type `FHIRReference` (represented as `dict` in JSON). """

        self.mode = None
        """ working | snapshot | changes.
        Type `str`. """

        self.orderedBy = None
        """ Order of section entries.
        Type `CodeableConcept` (represented as `dict` in JSON). """

        self.section = None
        """ Nested Section.
        List of `CompositionSection` items (represented as `dict` in JSON). """

        self.text = None
        """ Text summary of the section, for human interpretation.
        Type `Narrative` (represented as `dict` in JSON). """

        self.title = None
        """ Label for section (e.g. for ToC).
        Type `str`. """

        super(CompositionSection, self).__init__(jsondict=jsondict, strict=strict)

    def elementProperties(self):
        js = super(CompositionSection, self).elementProperties()
        js.extend([
            ("author", "author", fhirreference.FHIRReference, "Reference", True, None, False),
            ("code", "code", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("emptyReason", "emptyReason", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("entry", "entry", fhirreference.FHIRReference, "Reference", True, None, False),
            ("focus", "focus", fhirreference.FHIRReference, "Reference", False, None, False),
            ("mode", "mode", str, "code", False, None, False),
            ("orderedBy", "orderedBy", codeableconcept.CodeableConcept, "CodeableConcept", False, None, False),
            ("section", "section", CompositionSection, "CompositionSection", True, None, False),
            ("text", "text", narrative.Narrative, "Narrative", False, None, False),
            ("title", "title", str, "string", False, None, False),
        ])
        return js


import sys
try:
    from . import codeableconcept
except ImportError:
    codeableconcept = sys.modules[__package__ + '.codeableconcept']
try:
    from . import fhirdate
except ImportError:
    fhirdate = sys.modules[__package__ + '.fhirdate']
try:
    from . import fhirreference
except ImportError:
    fhirreference = sys.modules[__package__ + '.fhirreference']
try:
    from . import identifier
except ImportError:
    identifier = sys.modules[__package__ + '.identifier']
try:
    from . import narrative
except ImportError:
    narrative = sys.modules[__package__ + '.narrative']
try:
    from . import period
except ImportError:
    period = sys.modules[__package__ + '.period']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  Generated from FHIR 3.0.1.11917 on 2019-05-13.
#  2019, SMART Health IT.

import os
import pytest
import io
import unittest
import json

from .fixtures import force_bytes
from .. import relatedperson
from ..fhirdate import FHIRDate


@pytest.mark.usefixtures("base_settings")
class RelatedPersonTests(unittest.TestCase):
    def instantiate_from(self, filename):
        datadir = os.environ.get('FHIR_UNITTEST_DATADIR') or ''
        with io.open(os.path.join(datadir, filename), 'r', encoding='utf-8') as handle:
            js = json.load(handle)
            self.assertEqual("RelatedPerson", js["resourceType"])
        return relatedperson.RelatedPerson(js)
    
    def testRelatedPerson1(self):
        inst = self.instantiate_from("relatedperson-example.json")
        self.assertIsNotNone(inst, "Must have instantiated a RelatedPerson instance")
        self.implRelatedPerson1(inst)
        
        js = inst.as_json()
        self.assertEqual("RelatedPerson", js["resourceType"])
        inst2 = relatedperson.RelatedPerson(js)
        self.implRelatedPerson1(inst2)
    
    def implRelatedPerson1(self, inst):
        self.assertTrue(inst.active)
        self.assertEqual(force_bytes(inst.address[0].city), force_bytes("Paris"))
        self.assertEqual(force_bytes(inst.address[0].country), force_bytes("FRA"))
        self.assertEqual(force_bytes(inst.address[0].line[0]), force_bytes("43, Place du Marché Sainte Catherine"))
        self.assertEqual(force_bytes(inst.address[0].postalCode), force_bytes("75004"))
        self.assertEqual(force_bytes(inst.gender), force_bytes("female"))
        self.assertEqual(force_bytes(inst.id), force_bytes("benedicte"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("urn:oid:1.2.250.1.61"))
        self.assertEqual(force_bytes(inst.identifier[0].type.text), force_bytes("INSEE"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("usual"))
        self.assertEqual(force_bytes(inst.identifier[0].value), force_bytes("272117510400399"))
        self.assertEqual(force_bytes(inst.name[0].family), force_bytes("du Marché"))
        self.assertEqual(force_bytes(inst.name[0].given[0]), force_bytes("Bénédicte"))
        self.assertEqual(force_bytes(inst.photo[0].contentType), force_bytes("image/jpeg"))
        self.assertEqual(force_bytes(inst.photo[0].url), force_bytes("Binary/f016"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].code), force_bytes("N"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0131"))
        self.assertEqual(force_bytes(inst.relationship.coding[1].code), force_bytes("WIFE"))
        self.assertEqual(force_bytes(inst.relationship.coding[1].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("+33 (237) 998327"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testRelatedPerson2(self):
        inst = self.instantiate_from("relatedperson-example-f001-sarah.json")
        self.assertIsNotNone(inst, "Must have instantiated a RelatedPerson instance")
        self.implRelatedPerson2(inst)
        
        js = inst.as_json()
        self.assertEqual("RelatedPerson", js["resourceType"])
        inst2 = relatedperson.RelatedPerson(js)
        self.implRelatedPerson2(inst2)
    
    def implRelatedPerson2(self, inst):
        self.assertEqual(force_bytes(inst.gender), force_bytes("female"))
        self.assertEqual(force_bytes(inst.id), force_bytes("f001"))
        self.assertEqual(force_bytes(inst.identifier[0].system), force_bytes("urn:oid:2.16.840.1.113883.2.4.6.3"))
        self.assertEqual(force_bytes(inst.identifier[0].type.text), force_bytes("BSN"))
        self.assertEqual(force_bytes(inst.identifier[0].use), force_bytes("official"))
        self.assertEqual(force_bytes(inst.name[0].family), force_bytes("Abels"))
        self.assertEqual(force_bytes(inst.name[0].given[0]), force_bytes("Sarah"))
        self.assertEqual(force_bytes(inst.name[0].use), force_bytes("usual"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].code), force_bytes("SIGOTHR"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("mobile"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("0690383372"))
        self.assertEqual(force_bytes(inst.telecom[1].system), force_bytes("email"))
        self.assertEqual(force_bytes(inst.telecom[1].use), force_bytes("home"))
        self.assertEqual(force_bytes(inst.telecom[1].value), force_bytes("s.abels@kpn.nl"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testRelatedPerson3(self):
        inst = self.instantiate_from("relatedperson-example-peter.json")
        self.assertIsNotNone(inst, "Must have instantiated a RelatedPerson instance")
        self.implRelatedPerson3(inst)
        
        js = inst.as_json()
        self.assertEqual("RelatedPerson", js["resourceType"])
        inst2 = relatedperson.RelatedPerson(js)
        self.implRelatedPerson3(inst2)
    
    def implRelatedPerson3(self, inst):
        self.assertEqual(force_bytes(inst.address[0].city), force_bytes("PleasantVille"))
        self.assertEqual(force_bytes(inst.address[0].line[0]), force_bytes("534 Erewhon St"))
        self.assertEqual(force_bytes(inst.address[0].postalCode), force_bytes("3999"))
        self.assertEqual(force_bytes(inst.address[0].state), force_bytes("Vic"))
        self.assertEqual(force_bytes(inst.address[0].use), force_bytes("home"))
        self.assertEqual(force_bytes(inst.gender), force_bytes("male"))
        self.assertEqual(force_bytes(inst.id), force_bytes("peter"))
        self.assertEqual(force_bytes(inst.name[0].family), force_bytes("Chalmers"))
        self.assertEqual(force_bytes(inst.name[0].given[0]), force_bytes("Peter"))
        self.assertEqual(force_bytes(inst.name[0].given[1]), force_bytes("James"))
        self.assertEqual(force_bytes(inst.name[0].use), force_bytes("official"))
        self.assertEqual(inst.period.start.date, FHIRDate("2012-03-11").date)
        self.assertEqual(inst.period.start.as_json(), "2012-03-11")
        self.assertEqual(force_bytes(inst.photo[0].contentType), force_bytes("image/jpeg"))
        self.assertEqual(force_bytes(inst.photo[0].url), force_bytes("Binary/f012"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].code), force_bytes("C"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].system), force_bytes("http://hl7.org/fhir/v2/0131"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("work"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("(03) 5555 6473"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))
    
    def testRelatedPerson4(self):
        inst = self.instantiate_from("relatedperson-example-f002-ariadne.json")
        self.assertIsNotNone(inst, "Must have instantiated a RelatedPerson instance")
        self.implRelatedPerson4(inst)
        
        js = inst.as_json()
        self.assertEqual("RelatedPerson", js["resourceType"])
        inst2 = relatedperson.RelatedPerson(js)
        self.implRelatedPerson4(inst2)
    
    def implRelatedPerson4(self, inst):
        self.assertEqual(inst.birthDate.date, FHIRDate("1963").date)
        self.assertEqual(inst.birthDate.as_json(), "1963")
        self.assertEqual(force_bytes(inst.gender), force_bytes("female"))
        self.assertEqual(force_bytes(inst.id), force_bytes("f002"))
        self.assertEqual(force_bytes(inst.name[0].text), force_bytes("Ariadne Bor-Jansma"))
        self.assertEqual(force_bytes(inst.name[0].use), force_bytes("usual"))
        self.assertEqual(inst.period.start.date, FHIRDate("1975").date)
        self.assertEqual(inst.period.start.as_json(), "1975")
        self.assertEqual(force_bytes(inst.photo[0].contentType), force_bytes("image/jpeg"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].code), force_bytes("SIGOTHR"))
        self.assertEqual(force_bytes(inst.relationship.coding[0].system), force_bytes("http://hl7.org/fhir/v3/RoleCode"))
        self.assertEqual(force_bytes(inst.telecom[0].system), force_bytes("phone"))
        self.assertEqual(force_bytes(inst.telecom[0].use), force_bytes("home"))
        self.assertEqual(force_bytes(inst.telecom[0].value), force_bytes("+31201234567"))
        self.assertEqual(force_bytes(inst.text.status), force_bytes("generated"))

